<?php namespace CookiePlusAddon\Includes;

class AssetsLoader {
    protected static $instance = null;
    public $plugin;
    public $admin_url = '';
    public $public_url = '';

    /*
    |---------------------------------------------------------------------------------------------------
    | Singleton
    |---------------------------------------------------------------------------------------------------
    */
    private function __clone(){
    }//Stopping Clonning of Object

    private function __wakeup(){
    }//Stopping unserialize of object

    public static function get_instance( $plugin = null ){
        if( null === self::$instance ){
            self::$instance = new self( $plugin );
        }
        return self::$instance;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Constructor
    |---------------------------------------------------------------------------------------------------
    */
    private function __construct( $plugin ){
        $this->plugin = $plugin;
        $this->admin_url = CPMPP_URL . 'assets/admin/';
        $this->public_url = CPMPP_URL . 'assets/public/';
        $this->libs_url = CPMPP_URL . 'libs/';

        add_action( 'admin_enqueue_scripts', array( $this, 'add_base_script' ), 1 );
        add_action( 'wp_enqueue_scripts', array( $this, 'add_base_script' ), 1 );

        add_action( 'admin_enqueue_scripts', array( $this, 'add_admin_scripts' ) );
        add_action( 'admin_enqueue_scripts', array( $this, 'add_admin_styles' ) );
        add_action( 'wp_enqueue_scripts', array( $this, 'add_public_scripts' ) );
        add_action( 'wp_enqueue_scripts', array( $this, 'add_public_styles' ) );

        add_filter( "autoptimize_filter_js_exclude", array( $this, 'autoptimize_filter_js_exclude' ), 10, 1 );


    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Add admin scripts
    |---------------------------------------------------------------------------------------------------
    */
    public function add_admin_scripts( $hook ){
        if( ! $this->should_add() ){
            return;
        }

        wp_register_script( 'admin-cookieplus-addon', $this->admin_url . 'js/admin-cookieplus-addon.js', array( 'jquery' ), CPMPP_VERSION );
        wp_enqueue_script( 'admin-cookieplus-addon' );

        $this->add_public_scripts();
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Add admin styles
    |---------------------------------------------------------------------------------------------------
    */
    public function add_admin_styles( $hook ){
        if( ! $this->should_add() ){
            return;
        }

        wp_register_style( 'admin-cookieplus-addon', $this->admin_url . 'css/admin-cookieplus-addon.css', array(), CPMPP_VERSION );
        wp_enqueue_style( 'admin-cookieplus-addon' );

        $this->add_public_styles();
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Comprueba si se debe agregar scripts y estilos
    |---------------------------------------------------------------------------------------------------
    */
    private function should_add(){
        global $current_screen;
        $load_in = array(
            $this->plugin->arg( 'masterpopups', 'post_type' ),
        );
        if( in_array( $current_screen->post_type, $load_in ) ){
            return true;
        }
        return false;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Add public scripts
    |---------------------------------------------------------------------------------------------------
    */
    public function add_public_scripts(){
        wp_register_script( 'tabster', $this->libs_url . 'tabster/js/tabster.js', array( 'jquery' ), CPMPP_VERSION );
        wp_enqueue_script( 'tabster' );

        wp_register_script( 'cookieplus-addon-switcher', $this->libs_url . 'xbox-switcher/xbox-switcher-.js', array(), CPMPP_VERSION );
        wp_enqueue_script( 'cookieplus-addon-switcher' );

        wp_register_script( 'cookieplus-addon', $this->public_url . 'js/cookieplus-addon-.js', array( 'jquery' ), CPMPP_VERSION, false );
        wp_enqueue_script( 'cookieplus-addon' );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Add public styles
    |---------------------------------------------------------------------------------------------------
    */
    public function add_public_styles(){
        wp_register_style( 'tabster', $this->libs_url . 'tabster/css/tabster.css', array(), CPMPP_VERSION );
        wp_enqueue_style( 'tabster' );

        wp_register_style( 'cookieplus-addon-switcher', $this->libs_url . 'xbox-switcher/xbox-switcher.css', array(), CPMPP_VERSION );
        wp_enqueue_style( 'cookieplus-addon-switcher' );


        wp_register_style( 'cookieplus-addon', $this->public_url . 'css/cookieplus-addon.css', array(), CPMPP_VERSION );
        wp_enqueue_style( 'cookieplus-addon' );

    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Add base cript
    |---------------------------------------------------------------------------------------------------
    */
    public function add_base_script(){
        wp_register_script( 'cookieplus-addon-cookie-blocker', CPMPP_URL . 'assets/public/js/cookieplus-addon-blocker.js', array(), CPMPP_VERSION, false );
        wp_enqueue_script( 'cookieplus-addon-cookie-blocker' );

        wp_localize_script( 'cookieplus-addon-cookie-blocker', 'CookiePlus', array(
            'CookieConsent' => $this->plugin->CookieConsent->get_options( true ),
            'TermsConditions' => $this->plugin->TermsConditions->get_public_options(),
            'PrivacyPolicy' => $this->plugin->PrivacyPolicy->get_public_options(),
        ) );

        wp_localize_script( 'cookieplus-addon-cookie-blocker', 'CookiePlusAddonSettings', array(
            'version' => CPMPP_VERSION,
            'ajax_url' => admin_url( 'admin-ajax.php' ),
            'ajax_nonce' => wp_create_nonce( 'mpp_ajax_nonce' ),
            'is_admin' => is_admin(),
            'disable_services' => array(
                'google_analytics' => array(
                    'disable' => $this->plugin->CookieConsent->maybe_disable_google_analytics(),
                    'id' => Options::option( 'google_analytics-id' ),
                )
            ),
            'texts' => array(
                'cookie_active' => Options::option( 'texts-cookie-on' ),
                'cookie_inactive' => Options::option( 'texts-cookie-off' ),
            ),
            'countries_info' => $this->plugin->CookieConsent->get_countries_info(),
            'reload' => Options::option( 'reload-on-save-cookies' ),
        ) );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Exclude js files for Autoptimize plugin
    |---------------------------------------------------------------------------------------------------
    */
    public function autoptimize_filter_js_exclude( $exclude ){
        return $exclude.", cookieplus-addon-blocker.js";
    }

}
