<?php namespace CookiePlusAddon\Includes;

use CookiePlusAddon\Includes\Integrations as Integrations;

class CookiePlusAddon {
    protected static $instance = null;
    public static $args = array();
    public $Options = null;
    public $SettingsPanel = null;
    public $TabCookieSettings = null;
    public $TermsConditions = null;
    public $PrivacyPolicy = null;
    public $CookieConsent = null;

    /*
	|---------------------------------------------------------------------------------------------------
	| Singleton
	|---------------------------------------------------------------------------------------------------
	*/
    private function __clone(){
    }//Stopping Clonning of Object

    private function __wakeup(){
    }//Stopping unserialize of object

    public static function get_instance( $version = '1.0.0' ){
        if( null === self::$instance ){
            self::$instance = new self( $version );

        }
        return self::$instance;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Constructor
	|---------------------------------------------------------------------------------------------------
	*/
    private function __construct( $args = array() ){
        self::$args = $args;

        $this->constants();
        $this->plugin_loader();
        $this->hooks();

    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Debug
	|---------------------------------------------------------------------------------------------------
	*/
    public function d( $var1, $var2 = null ){
        echo '<pre style="margin-left: 200px; margin-top: 20px;">';
        print_r( $var1 );
        echo '</pre>';
        if( $var2 !== null ){
            echo '<pre style="margin-left: 200px; margin-top: 20px;">';
            print_r( $var2 );
            echo '</pre>';
        }
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Plugin arguments
	|---------------------------------------------------------------------------------------------------
	*/
    public function arg( $name = '', $key = '' ){
        if( isset( self::$args[$name] ) ){
            if( $key ){
                if( isset( self::$args[$name][$key] ) ){
                    return self::$args[$name][$key];
                } else{
                    return null;
                }
            }
            return self::$args[$name];
        }
        return null;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Constants
	|---------------------------------------------------------------------------------------------------
	*/
    public function constants(){
        define( 'CPMPP_VERSION', $this->arg( 'version' ) );
        define( 'CPMPP_DIR', trailingslashit( dirname( dirname( __FILE__ ) ) ) );
        define( 'CPMPP_URL', trailingslashit( plugins_url( '', dirname( __FILE__ ) ) ) );
        define( 'CPMPP_TOTAL_GROUPS_COOKIE_CONSENT', 5 );
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Verifica si Master Popups está activo
	|---------------------------------------------------------------------------------------------------
	*/
    public function is_masterpopups_active(){
        if( ! function_exists( 'is_plugin_active' ) ){
            include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
        }
        return is_plugin_active( 'master-popups/master-popups.php' ) || is_plugin_active( 'master-popups-lite/master-popups.php' );
    }



    /*
	|---------------------------------------------------------------------------------------------------
	| Plugin loader
	|---------------------------------------------------------------------------------------------------
	*/
    private function plugin_loader(){
        include dirname( __FILE__ ) . '/class-autoloader.php';
        include CPMPP_DIR . 'libs/GeoIP2-php/vendor/autoload.php';
        ClassAutoloader::run();

        if( ! $this->is_masterpopups_active() ){
            return;
        }

        AssetsLoader::get_instance( $this );
        $this->SettingsPanel = SettingsPanel::get_instance( $this );
        $this->Options = Options::get_instance( $this );
        $this->CookieConsent = CookieConsent::get_instance( $this );
        $this->TabCookieSettings = TabCookieSettings::get_instance( $this );
        $this->TermsConditions = TermsConditions::get_instance( $this );
        $this->PrivacyPolicy = PrivacyPolicy::get_instance( $this );

        Integrations\WpComments::get_instance( $this );
        Integrations\Woocommerce::get_instance( $this );
        Integrations\Buddypress::get_instance( $this );
        Integrations\CF7::get_instance( $this );
        Integrations\GravityForms::get_instance( $this );
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Plugin hooks
	|---------------------------------------------------------------------------------------------------
	*/
    private function hooks(){
        add_action( 'plugins_loaded', array( $this, 'plugin_textdomain' ) );
        add_action( 'admin_notices', array( $this, 'show_message_to_install_masterpopups' ) );
        add_filter( "mpp_add_popup_templates", array( $this, 'add_popup_templates' ), 10, 2 );

        add_action( 'wp_head', array( $this, 'load_resources_in_header' ), 10 );
        add_action( 'admin_head', array( $this, 'load_resources_in_header' ), 10 );
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Plugin text domain
	|---------------------------------------------------------------------------------------------------
	*/
    public function plugin_textdomain(){
        $plugin_rel_path = trailingslashit ( plugin_basename( CPMPP_DIR ) );
        $loaded = load_plugin_textdomain( 'mpp-cookieplus', false, $plugin_rel_path. 'languages/' );

        if( ! $loaded ){
            load_textdomain( 'mpp-cookieplus', CPMPP_DIR . 'languages/cookieplus-' . get_locale() . '.mo' );
        }
    }


    /*
	|---------------------------------------------------------------------------------------------------
	| Cookie Plus Popup Templates
	|---------------------------------------------------------------------------------------------------
	*/
    public function show_message_to_install_masterpopups(){
        if( ! $this->is_masterpopups_active() ){
            $message = sprintf( 'You have to install and activate %sMaster Popups%s plugin before using %s"Cookie Plus"%s', '<a href="http://bit.ly/2HQjoKF" target="_blank"><strong>', '</strong></a>', '<strong>', '</strong>' );
            $print = "<h3 style='margin: 10px 0px; font-size:1.1em;'>Missing plugin: Master Popups</h3>";
            $print .= "<p>$message</p>";
            echo "<div class='notice notice-warning update-nag' style='display:block;'>$print</div>";
        } else if( version_compare( MPP_VERSION, '2.0.0', '<' ) ) {
            $message = sprintf( 'You have an old version of %sMaster Popups%s you need version 2.0.0 or higher to use %s"Cookie Plus"%s', '<a href="http://bit.ly/2HQjoKF" target="_blank"><strong>', '</strong></a>', '<strong>', '</strong>' );
            $print = "<h3 style='margin: 10px 0px; font-size:1.1em;'>{$this->arg('menu_title')} Message:</h3>";
            $print .= "<p>$message</p>";
            echo "<div class='notice notice-warning update-nag' style='display:block;'>$print</div>";
        }
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Carga de recursos en header
	|---------------------------------------------------------------------------------------------------
	*/
    public function load_resources_in_header(){
        if( ! $this->is_masterpopups_active() ){
            return;
        }
        $return = "";
        $version = $this->arg( 'version' );
        $return .= "\n\n<!-- MPP:CookiePlus v$version -->";
        $return .= "\n\n<!-- [BEGIN] MPP:CookiePlus:Header Styles -->";
        $return .= "\n<style>";
        $return .= "\n/* Custom CSS - Tab Cookie Settings */\n";
        $return .= $this->TabCookieSettings->build_style();
        $return .= "\n/* Custom CSS - General Settings */\n";
        $return .= Options::option( 'custom-css' );
        $return .= "\n</style>";

        $return .= "\n<!-- [END] MPP:CookiePlus:Header Styles -->\n\n";
        echo $return;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Retorna las cookies usadas en el plugin
	|---------------------------------------------------------------------------------------------------
	*/
    public function get_plugin_cookies(){
        return array(
            Options::option( 'cookie-consent-cookie-name' ),
            Options::option( 'terms-conditions-cookie-name' ),
            Options::option( 'privacy-policy-cookie-name' ),
        );
    }


    /*
	|---------------------------------------------------------------------------------------------------
	| Cookie Plus Popup Templates
	|---------------------------------------------------------------------------------------------------
	*/
    public function add_popup_templates( $original_templates = array(), $version = '1.0.0' ){
        $templates_url = CPMPP_URL . 'assets/templates/';
        return array(
            array(
                'template' => $templates_url . 'popup-template-001.json',
                'image' => $templates_url . 'popup-template-001.jpg',
                'type' => 'modal',
                'category' => 'general',
                'tags' => 'cookies, cookie policy, GDPR, cookie consent',
            ),
            array(
                'template' => $templates_url . 'popup-template-002.json',
                'image' => $templates_url . 'popup-template-002.jpg',
                'type' => 'modal',
                'category' => 'general',
                'tags' => 'cookies, cookie policy, GDPR, cookie consent',
            ),
            array(
                'template' => $templates_url . 'popup-template-003.json',
                'image' => $templates_url . 'popup-template-003.jpg',
                'type' => 'modal',
                'category' => 'general',
                'tags' => 'cookies, cookie policy, GDPR, cookie consent',
            ),
            array(
                'template' => $templates_url . 'popup-template-004.json',
                'image' => $templates_url . 'popup-template-004.jpg',
                'type' => 'modal',
                'category' => 'general',
                'tags' => 'cookies, cookie policy, GDPR, cookie consent',
            ),
            array(
                'template' => $templates_url . 'popup-template-005.json',
                'image' => $templates_url . 'popup-template-005.jpg',
                'type' => 'modal',
                'category' => 'general',
                'tags' => 'cookies, cookie policy, GDPR, cookie consent',
            ),
            array(
                'template' => $templates_url . 'popup-template-006.json',
                'image' => $templates_url . 'popup-template-006.jpg',
                'type' => 'modal',
                'category' => 'general',
                'tags' => 'cookies, cookie policy, GDPR, cookie consent',
            ),
        );

    }
}