<?php namespace CookiePlusAddon\Includes;

class Functions {
    /*
    |---------------------------------------------------------------------------------------------------
    | Get post id
    |---------------------------------------------------------------------------------------------------
    */
    public static function post_id(){
        $post_id = 0;
        if( ! is_admin() ){
            $post_id = get_the_ID();
        } else{
            if( Functions::is_post_page( 'new' ) ){
                return 0;
            }
            if( ! $post_id ){
                $post_id = isset( $_REQUEST['post'] ) ? $_REQUEST['post'] : $post_id;
            }
            if( ! $post_id ){
                $post_id = isset( $_GET['post'] ) ? $_GET['post'] : $post_id;
            }
            if( ! $post_id ){
                $post_id = isset( $GLOBALS['post']->ID ) ? $GLOBALS['post']->ID : 0;
            }
        }

        if( is_array( $post_id ) ){
            return 0;
        }
        return $post_id;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Comprueba si un string empieza con un caracter específico
    |---------------------------------------------------------------------------------------------------
    */
    public static function starts_with( $needle, $haystack, $case_sensitive = false ){
        if( strlen( $needle ) == 0 || strlen( $haystack ) == 0 ){
            return false;
        }
        return substr_compare( $haystack, $needle, 0, strlen( $needle ), ! $case_sensitive ) === 0;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Comprueba si un string termina con un caracter específico
    |---------------------------------------------------------------------------------------------------
    */
    public static function ends_with( $needle, $haystack, $case_sensitive = false ){
        $offset = strlen( $haystack ) - strlen( $needle );
        if( strlen( $needle ) == 0 || strlen( $haystack ) == 0 || $offset >= strlen( $haystack ) ){
            return false;
        }
        return substr_compare( $haystack, $needle, $offset, strlen( $needle ), ! $case_sensitive ) === 0;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Ordena un array
    |---------------------------------------------------------------------------------------------------
    */
    public static function sort( &$array = array(), $sort = 'asc' ){
        if( strtolower( $sort ) == 'asc' ){
            ksort( $array );
        } elseif( strtolower( $sort ) == 'desc' ){
            krsort( $array );
        }
        return $array;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Comprueba si un valor existe en un array
    |---------------------------------------------------------------------------------------------------
    */
    public static function in_array( $find, $array = array(), $search_asterisk = false ){
        if( in_array( $find, $array ) ){
            return true;
        }
        $exists = false;
        if( $search_asterisk ){
            foreach( $array as $key => $value ){
                if( substr( $value, -1 ) == '*' ){
                    if( strpos( $find, substr( $value, 0, strlen( $value ) - 1 ) ) !== false ){
                        $exists = true;
                        break;
                    }
                }
            }
        }
        return $exists;
    }


    /*
    |---------------------------------------------------------------------------------------------------
    | Comprueba si una variable está vacía
    |---------------------------------------------------------------------------------------------------
    */
    public static function is_empty( $value = '' ){
        if( is_array( $value ) ){
            $value = array_filter( $value );
            if( empty( $value ) ){
                return true;
            }
            return false;
        } else if( is_numeric( $value ) ){
            return false;
        } else if( empty( $value ) ){
            return true;
        } else{
            return false;
        }
    }


    /*
    |---------------------------------------------------------------------------------------------------
    | Random string
    |---------------------------------------------------------------------------------------------------
    */
    public static function random_string( $length = 10, $numbers = true ){
        $str = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $str = $numbers ? $str . '0123456789' : $str;
        return substr( str_shuffle( $str ), 0, $length );
    }


    /*
    |---------------------------------------------------------------------------------------------------
    | Elimina espacios en blanco en un texto y lo convierte a minuscula
    |---------------------------------------------------------------------------------------------------
    */
    public static function str_trim_to_lower( $string, $replace = '-' ){
        $string = strtolower( $string );
        $string = preg_replace( '/[_]+/', '_', $string );
        $string = preg_replace( '/[\s-]+/', $replace, $string );
        return $string;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Comprueba si estamos en una página opciones. Solo funciona con páginas dentro de admin.php?page=*
    |---------------------------------------------------------------------------------------------------
    */
    public static function is_admin_page(){
        global $pagenow;
        if( ! is_admin() ){
            return false;
        }
        return $pagenow == 'admin.php';
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Comprueba si estamos en la página de inicio de sesión o de registro
    |---------------------------------------------------------------------------------------------------
    */
    public static function is_login_page(){
        return in_array( $GLOBALS['pagenow'], array( 'wp-login.php', 'wp-register.php' ) );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Comprueba si estamos en una página opciones
    |---------------------------------------------------------------------------------------------------
    */
    public static function is_post_page( $page = '' ){
        global $pagenow;
        if( ! is_admin() ){
            return false;
        }
        if( $page == 'edit' ){
            return in_array( $pagenow, array( 'post.php' ) );
        } elseif( $page == 'new' ){
            return in_array( $pagenow, array( 'post-new.php' ) );
        }
        return in_array( $pagenow, array( 'post.php', 'post-new.php' ) );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Comprueba si estamos en una página admin de un post_type
    |---------------------------------------------------------------------------------------------------
    */
    public static function is_admin_post_type_page( $post_type = '' ){
        $screen = get_current_screen();
        if( is_admin() && $screen && isset( $screen->parent_file ) ){
            return $screen->parent_file == 'edit.php?post_type=' . $post_type;
        }
        return false;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Comprueba si estamos en la página principal
    |---------------------------------------------------------------------------------------------------
    */
    public static function is_homepage(){
        $is_homepage = false;
        if( is_front_page() && is_home() ){
            $is_homepage = true; //Default homepage
        } elseif( is_front_page() ){
            $is_homepage = true; //Static homepage
        } elseif( is_home() ){
            $is_homepage = true; //Blog page
        } elseif( $_SERVER["REQUEST_URI"] == '/' ){
            $is_homepage = true;
        }
        return $is_homepage;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Comprueba si es localhost
    |---------------------------------------------------------------------------------------------------
    */
    public static function is_localhost(){
        return in_array( strtolower( $_SERVER['SERVER_NAME'] ), array( 'localhost', '127.0.0.1' ) );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Get current url
    |---------------------------------------------------------------------------------------------------
    */
    public static function current_url( $end_slash = true ){
        global $wp;
        $current_url = home_url( add_query_arg( array(), $wp->request ) );
        return $end_slash ? $current_url . '/' : $current_url;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Url admin de un post type
    |---------------------------------------------------------------------------------------------------
    */
    public static function post_type_url( $post_type, $action = 'edit', $query = array() ){
        $action = $action == 'new' ? 'post-new.php?post_type=' : 'edit.php?post_type=';
        $end_url = is_array( $query ) && ! empty( $query ) ? '&' . http_build_query( $query ) : '';
        return get_admin_url() . $action . $post_type . $end_url;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Propiedad From para header de un Email de Wordpress
    |---------------------------------------------------------------------------------------------------
    */
    public static function from_email( $from = 'wordpress' ){
        $admin_email = get_option( 'admin_email' );
        $sitename = strtolower( $_SERVER['SERVER_NAME'] );

        if( self::is_localhost() ){
            return $admin_email;
        }

        if( substr( $sitename, 0, 4 ) == 'www.' ){
            $sitename = substr( $sitename, 4 );
        }

        if( strpbrk( $admin_email, '@' ) == '@' . $sitename ){
            return $admin_email;
        }

        return $from . '@' . $sitename;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Convierte un string a CamelCase
    |---------------------------------------------------------------------------------------------------
    */
    public static function string_to_camelcase( $string, $capitalize = true ){
        $normalize_string = str_replace( array( '-', '_' ), ' ', $string );
        $ucwords = ucwords( $normalize_string, ' ' );
        $camelcase_string = str_replace( ' ', '', $ucwords );
        if( ! $capitalize ){
            $camelcase_string = lcfirst( $camelcase_string );
        }
        return $camelcase_string;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Devuelve el dominio de una url
    |---------------------------------------------------------------------------------------------------
    */
    public static function url_to_domain( $url, $length = 50 ){
        $host = parse_url( $url, PHP_URL_HOST );
        //If the URL can't be parsed, use the original URL
        if( ! $host ){
            $host = $url;
        }
        //Remove www.
        if( substr( $host, 0, 4 ) == 'www.' ){
            $host = substr( $host, 4 );
        }
        //Limit the domain length
        if( strlen( $host ) > $length ){
            $host = substr( $host, 0, $length - 3 ) . '...';
        }
        return $host;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Devuelve el dominio de una url
    |---------------------------------------------------------------------------------------------------
    */
    public static function days_to_seconds( $days ){
        return intval( $days ) * 24 * 60 * 60;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Comprueba si woocommerce está activado
    |---------------------------------------------------------------------------------------------------
    */
    public static function is_woocommerce_activated(){
        if( ! function_exists( 'is_plugin_active' ) ){
            include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
        }
        return class_exists( 'WooCommerce' ) || is_plugin_active( 'woocommerce/woocommerce.php' );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Comprueba si woocommerce está activado
    |---------------------------------------------------------------------------------------------------
    */
    public static function get_permalink( $page_id = '', $default = 'javascript: void(0)' ){
        $link = $default;
        if( $page_id && get_post( $page_id ) ){
            $link = get_permalink( $page_id );
        }
        return $link;
    }


}
