<?php namespace CookiePlusAddon\Includes;

abstract class GDPRPage {
    public $plugin = null;
    public $type = '';
    public $prefix = '';

    /*
    |---------------------------------------------------------------------------------------------------
    | Constructor
    |---------------------------------------------------------------------------------------------------
    */
    protected function __construct( $plugin, $type = 'TermsConditions' ){
        $this->plugin = $plugin;
        $this->type = $type;
        $this->set_prefix();
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Debug
    |---------------------------------------------------------------------------------------------------
    */
    protected function d( $var1, $var2 = null ){
        return $this->plugin->d( $var1, $var2  );
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Crea el shortcode para dar consentimiento
	|---------------------------------------------------------------------------------------------------
	*/
    abstract public function build_shortcode();

    /*
    |---------------------------------------------------------------------------------------------------
    | Comprueba si hay campos para procesar
    |---------------------------------------------------------------------------------------------------
    */
    public function set_prefix(){
        switch ( $this->type ) {
            case 'TermsConditions':
                $this->prefix = 'terms-conditions-';
                break;
            case 'PrivacyPolicy':
                $this->prefix = 'privacy-policy-';
                break;
        }
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Get option
	|---------------------------------------------------------------------------------------------------
	*/
    public function get_option( $option_name ){
        return Options::option( $this->prefix . $option_name );
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Public options
	|---------------------------------------------------------------------------------------------------
	*/
    public function get_public_options(){
        $options = array(
            'page_id' => $this->get_option( 'page' ),
            'page_url' => Functions::get_permalink( $this->get_option( 'page' ), '' ),
            'redirect_page' => $this->get_option( 'redirect-page' ),
            'cookie_name' => $this->get_option( 'cookie-name' ),
            'thanks_message' => $this->get_option( 'thanks-message' ),
            'consent_given' => $this->is_already_consent()
        );
        return $options;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Check version
	|---------------------------------------------------------------------------------------------------
	*/
    public function is_current_version( $data ){
        return ! empty( $data['version'] ) && $data['version'] == $this->get_option( 'version' );
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Check expiration time
	|---------------------------------------------------------------------------------------------------
	*/
    public function is_still_valid( $data ){
        return ! empty( $data['expiration_time'] ) && intval( $data['expiration_time'] ) > time();
    }


    /*
	|---------------------------------------------------------------------------------------------------
	| Establece el consentimiento
	|---------------------------------------------------------------------------------------------------
	*/
    public function set_consent(){
        $response = array(
            'success' => true,
        );

        $cookie_name = $this->get_option( 'cookie-name' );
        $cookie_duration = $this->get_option( 'cookie-duration' );
        $expiration_time = time() + Functions::days_to_seconds( $cookie_duration );

        $cookie_value = array(
            'page_id' => $this->get_option( 'page' ),
            'expiration_time' => $expiration_time,
            'version' => $this->get_option( 'version' ),
            'date_created' => current_time( 'mysql' ),
        );

        Cookie::set_encoded( $cookie_name, $cookie_value, $expiration_time, '/' );

        //Save user data
        $save_user_data = $this->get_option( 'save-userdata' );

        if ( $save_user_data == 'on' && is_user_logged_in() ) {
            update_user_meta( get_current_user_id(), $cookie_name, $cookie_value );
        }

        wp_send_json( $response );
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Comprueba si ya dio su consentimiento
	|---------------------------------------------------------------------------------------------------
	*/
    public function is_already_consent(){
        $accepted = false;
        $cookie_name = $this->get_option( 'cookie-name' );
        $cookie_value = Cookie::get( $cookie_name, true );

        if ( $cookie_value && is_array( $cookie_value ) ) {
            if ( $this->is_current_version( $cookie_value ) && $this->is_still_valid( $cookie_value ) ) {
                $accepted = true;
            }
        }
        return $accepted;
    }


}
