<?php namespace CookiePlusAddon\Includes;


class TermsConditions extends GDPRPage {
    protected static $instance = null;
    public $plugin;

    /*
	|---------------------------------------------------------------------------------------------------
	| Singleton
	|---------------------------------------------------------------------------------------------------
	*/
    private function __clone(){
    }//Stopping Clonning of Object

    private function __wakeup(){
    }//Stopping unserialize of object

    public static function get_instance( $plugin ){
        if( null === self::$instance ){
            self::$instance = new self( $plugin );
        }
        return self::$instance;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Constructor
	|---------------------------------------------------------------------------------------------------
	*/
    protected function __construct( $plugin ){
        $this->plugin = $plugin;
        parent::__construct( $plugin, 'TermsConditions' );
        $this->hooks();
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Plugin hooks
	|---------------------------------------------------------------------------------------------------
	*/
    private function hooks(){
        add_shortcode( 'cookieplus_btn_accept_terms', array( $this, 'build_shortcode' ) );
        add_filter( 'the_content', array( $this, 'insert_shortcode_automatically' ), 11 );
        add_action( 'wp_ajax_cookieplus_accept_terms', array( $this, 'set_consent' ) );
        add_action( 'wp_ajax_nopriv_cookieplus_accept_terms', array( $this, 'set_consent' ) );

    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Build shortcode
	|---------------------------------------------------------------------------------------------------
	*/
    public function build_shortcode( $atts = array(), $content = null ){
        $start = '<div class="cookieplus">';
        $end = '</div>';

        if( $this->is_already_consent() ){
            $message = '<div class="cookieplus-accepted-message cookieplus-terms-accepted-message">' . $this->get_option( 'thanks-message' ) . '</div>';
            return $start . $message . $end;
        }

        $atts = shortcode_atts( array(
            'id' => '',
            'class' => '',
            'radius' => '5px',
            'style' => '',
        ), $atts );

        $button = '';
        $id = ! empty( $atts['id'] ) ? "id='{$atts['id']}'" : '';
        $class = 'cookieplus-btn cookieplus-btn-green ' . $atts['class'];
        $button .= "<div $id class='cookieplus-btn-accept-terms-conditions $class' style='border-radius: {$atts['radius']}; {$atts['style']}'>";
        $button .= do_shortcode( $content );
        $button .= "</div>";

        return $start . $button . $end;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Insert shortcode automatically
	|---------------------------------------------------------------------------------------------------
	*/
    public function insert_shortcode_automatically( $content ){
        $page_id = $this->get_option( 'page' );
        $shortcode_auto = $this->get_option( 'shortcode-auto' );
        if( $shortcode_auto == 'on' && $page_id && $page_id == get_the_ID() ){
            $text = $this->get_option( 'shortcode-auto-text' );
            $shortcode = do_shortcode( '[cookieplus_btn_accept_terms]' . $text . '[/cookieplus_btn_accept_terms]' );
            return $content . $shortcode;
        }
        return $content;
    }


}