<?php namespace CookiePlusAddon\Includes\Integrations;

use CookiePlusAddon\Includes\Options;
use CookiePlusAddon\Includes\Functions;

class GravityForms {
    protected static $instance = null;
    public $plugin;
    public $checkbox_id = 1298;

    /*
	|---------------------------------------------------------------------------------------------------
	| Singleton
	|---------------------------------------------------------------------------------------------------
	*/
    private function __clone(){
    }//Stopping Clonning of Object

    private function __wakeup(){
    }//Stopping unserialize of object

    public static function get_instance( $plugin ){
        if( null === self::$instance ){
            self::$instance = new self( $plugin );
        }
        return self::$instance;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Constructor
	|---------------------------------------------------------------------------------------------------
	*/
    protected function __construct( $plugin ){
        $this->plugin = $plugin;
        $this->hooks();
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Plugin hooks
	|---------------------------------------------------------------------------------------------------
	*/
    private function hooks(){
        add_filter( 'gform_pre_render', array( $this, 'add_privacy_policy_checkbox' ), 999 );
        add_filter( 'gform_validation', array( $this, 'validate_privacy_policy_checkbox' ), 999 );
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Agrega checkbox al formulario
	|---------------------------------------------------------------------------------------------------
	*/
    public function add_privacy_policy_checkbox( $form ){
        if( Options::option( 'gravity-forms-enabled' ) == 'on' && isset( $form['fields'] ) ){
            $label = Options::option( 'gravity-forms-policy-label' );
            $text = Options::option( 'gravity-forms-policy-text' );
            $validation_text = Options::option( 'gravity-forms-policy-validation-text' );

            $page_id = Options::option( 'privacy-policy-page' );
            if ( $page_id > 0 ) {
                $text .= '<a href="' . Functions::get_permalink( $page_id ) . '" target="_blank">' .$label . '</a>';
            }

            $field_args = array(
                'id' => $this->checkbox_id,
                'type' => 'checkbox',
                'isRequired' => true,
                'label' => $label,
                'description' => '',
                'adminLabel' => '',
                'visibility' => 'visible',
                'size' => 'medium',
                'choices' => array(
                    array(
                        'text' => $text,
                        'value' => '1',
                        'isSelected' => false,
                    )
                ),
            );

            if( count( $_POST ) > 0 && ! isset( $_POST["input_{$this->checkbox_id}_1"] ) ){
                $field_args['failed_validation'] = true;
                $field_args['validation_message'] = $validation_text;
            }
            if( class_exists( 'GF_Fields' ) ){
                $field = \GF_Fields::create( $field_args );
                array_push( $form['fields'], $field );
            }
        }
        return $form;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Valida checkbox políticas de privacidad
	|---------------------------------------------------------------------------------------------------
	*/
    public function validate_privacy_policy_checkbox( $validation_result ){
        $form = $validation_result['form'];
        if( Options::option( 'gravity-forms-enabled' ) == 'on' ){
            $validation_text = Options::option( 'gravity-forms-policy-validation-text' );
            if( empty( $_POST["input_{$this->checkbox_id}_1"] ) ){
                $validation_result['is_valid'] = false;
                //finding Field with ID and marking it as failed validation
                foreach( $form['fields'] as &$field ){
                    if( $field->id == $this->checkbox_id ){
                        $field->failed_validation = true;
                        $field->validation_message = $validation_text;
                        break;
                    }
                }
            }
        }
        return $validation_result;
    }


}