<?php namespace CookiePlusAddon\Includes\Integrations;

use CookiePlusAddon\Includes\Options;
use CookiePlusAddon\Includes\Functions;

class Woocommerce {
    protected static $instance = null;
    public $plugin;

    /*
	|---------------------------------------------------------------------------------------------------
	| Singleton
	|---------------------------------------------------------------------------------------------------
	*/
    private function __clone(){
    }//Stopping Clonning of Object

    private function __wakeup(){
    }//Stopping unserialize of object

    public static function get_instance( $plugin ){
        if ( null === self::$instance ) {
            self::$instance = new self( $plugin );
        }
        return self::$instance;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Constructor
	|---------------------------------------------------------------------------------------------------
	*/
    protected function __construct( $plugin ){
        $this->plugin = $plugin;
        $this->hooks();
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Plugin hooks
	|---------------------------------------------------------------------------------------------------
	*/
    private function hooks(){
        add_filter( 'woocommerce_product_review_comment_form_args', array( $this, 'add_privacy_policy_checkbox_to_reviews' ), 10, 1 );
        add_action( 'woocommerce_checkout_after_terms_and_conditions', array( $this, 'add_privacy_policy_checkbox_to_checkout' ), 10 );
        add_action( 'woocommerce_register_form_end', array( $this, 'add_privacy_policy_checkbox_to_register' ), 10 );
        add_action( 'woocommerce_after_checkout_validation', array( $this, 'validate_privacy_policy_on_checkout' ), 10, 2 );
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Checkbox
	|---------------------------------------------------------------------------------------------------
	*/
    public function get_checkbox(){
        $checkbox = '';
        $text = Options::option( 'woocommerce-policy-text' );
        $page_id = Options::option( 'privacy-policy-page' );
        if ( $page_id > 0 ) {
            $text = '<a href="' . Functions::get_permalink( $page_id ) . '" target="_blank">' . $text . '</a>';
        }
        $checkbox .= '<label class="cookieplus-privacy-policy-label woocommerce-form__label woocommerce-form__label-for-checkbox checkbox">';
        $checkbox .= '<input type="checkbox" name="accept_privacy_policy" id="accept_privacy_policy" required="required" class="woocommerce-form__input woocommerce-form__input-checkbox input-checkbox">';
        $checkbox .= '<span>' . $text . '</span> <span class="required">*</span>';
        $checkbox .= '</label>';
        return $checkbox;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Agrega checkbox al sistema de comentarios de Woocommerce
	|---------------------------------------------------------------------------------------------------
	*/
    public function add_privacy_policy_checkbox_to_reviews( $comment_form ){
        if ( ! is_product() ) {
            return $comment_form;
        }
        $show_in = (array) Options::option( 'woocommerce-policy-show-in' );
        if ( Options::option( 'woocommerce-enabled' ) == 'on' && in_array( 'reviews', $show_in ) ) {
            $return = '<p class="cookieplus form-row terms wc-terms-and-conditions">';
            $return .= $this->get_checkbox();
            $return .= '</p>';
            $comment_form['comment_field'] .= $return;
        }
        return $comment_form;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Agrega checkbox a la página de pago de Woocommerce
	|---------------------------------------------------------------------------------------------------
	*/
    public function add_privacy_policy_checkbox_to_checkout(){
        $show_in = (array) Options::option( 'woocommerce-policy-show-in' );
        if ( Options::option( 'woocommerce-enabled' ) == 'on' && in_array( 'checkout', $show_in ) ) {
            $return = '<p class="cookieplus form-row terms wc-terms-and-conditions">';
            $return .= $this->get_checkbox();
            $return .= '</p>';
            echo $return;
        }
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Agrega checkbox a la página de registro (My account) de Woocommerce
	|---------------------------------------------------------------------------------------------------
	*/
    public function add_privacy_policy_checkbox_to_register(){
        $show_in = (array) Options::option( 'woocommerce-policy-show-in' );
        if ( Options::option( 'woocommerce-enabled' ) == 'on' && in_array( 'register', $show_in ) ) {
            $return = '<p class="cookieplus form-row terms wc-terms-and-conditions">';
            $return .= $this->get_checkbox();
            $return .= '</p>';
            echo $return;
        }
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Valida checkbox políticas de privacidad en la página de pago
	|---------------------------------------------------------------------------------------------------
	*/
    public function validate_privacy_policy_on_checkout( $data, $errors ){
        $show_in = (array) Options::option( 'woocommerce-policy-show-in' );
        if ( Options::option( 'woocommerce-enabled' ) == 'on' && in_array( 'checkout', $show_in ) ) {
            if ( ! isset( $_POST['accept_privacy_policy'] ) ) {
                $errors->add( 'privacy-policy', Options::option('woocommerce-policy-validation-text') );
            }
        }
    }


}